package com.codegeist.fisheye.domain;

import java.util.ArrayList;
import java.util.List;

import com.codegeist.fisheye.domain.query.Query.ReturnValue;

/**
 * Results of a query that contains a 'return' clause.
 * 
 * @author Brock Janiczak
 */
public class QueryResultData {
	private final ReturnValue[] columns;
	private List<Object[]> data;
	
	/**
	 * Creates a new result set with the expected columns.
	 * @param columns Columns that will be returned from the query
	 * @throws IllegalArgumentException columns is null
	 */
	public QueryResultData(ReturnValue[] columns) {
		if (columns == null || columns.length == 0) {
			throw new IllegalArgumentException("Columns must not be null or empty");
		}
		
		this.columns = columns;
		data = new ArrayList<Object[]>();
	}
	
	/**
	 * @return Details of the row data in this result set
	 */
	public ReturnValue[] getColumns() {
		return columns;
	}
	
	/**
	 * Adds a line of data to this result.  The data must contain one item for each column
	 * @param line Row of data to add to this result
	 * @throws IllegalArgumentException
	 * <ul>
	 * <li><code>null</code> data is supplied</li>
	 * <li>the number of entries in the row do not match the column specification</li>
	 * <li>The data types of the row data doesn't match the column specification</li>
	 * </ul>
	 */
	public void addLine(Object[] line) {
		if (line == null || line.length != columns.length) {
			throw new IllegalArgumentException("Data doesn't match column specification");
		}
		
		for (int i = 0; i < columns.length; i++) {
			if (line[i] != null) {
				if (!columns[i].getAttribute().getJavaType().equals(line[i].getClass())) {
					throw new IllegalArgumentException("Type of column " + columns[i].getColumnName() + ": " + line[i].getClass() + " doesn't match expected type: " + columns[i].getAttribute().getJavaType());
				}
			}
		}
		
		data.add(line);
	}
	
	public List<Object[]> getData() {
		return data;
	}
	
}
